/* 
  KanaLiquidCrysta.h
  
  This file is part of KanaLiquidCrystal library for Arduino.
  
  KanaLiquidCrystal library was developed by Synapse(Hiroshi Tanigawa) in 2013.
  This Library is originally distributed at "Synapse's history of making gadgets." 
  <http://www3.big.or.jp/~schaft/hardware/>

  KanaLiquidCrystal library is free software: you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License as published
  by the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.

  KanaLiquidCrystal library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with MGLCD.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KANALIQUIDCRYSTAL_H
#define KANALIQUIDCRYSTAL_H

#define KLCD_VERSION 100 // Version 1.00

#include <arduino.h>
#include "../../../../libraries/LiquidCrystal/LiquidCrystal.h"

class KanaLiquidCrystal : public LiquidCrystal {
private:
  typedef byte (*PDecodeFunc)(byte);
  PDecodeFunc DecodeChar;

  // Decode functions
  static byte DecodeStraight(byte ch) { return ch; }
  static byte DecodeUtf8(byte ch);  
public:
  // Override LiquidCristal's constructors
  KanaLiquidCrystal(uint8_t rs, uint8_t enable, uint8_t d0, uint8_t d1, uint8_t d2, uint8_t d3, uint8_t d4, uint8_t d5, uint8_t d6, uint8_t d7)
    : LiquidCrystal(rs,enable,d0,d1,d2,d3,d4,d5,d6,d7), DecodeChar(DecodeStraight) {}
  KanaLiquidCrystal(uint8_t rs, uint8_t rw, uint8_t enable, uint8_t d0, uint8_t d1, uint8_t d2, uint8_t d3, uint8_t d4, uint8_t d5, uint8_t d6, uint8_t d7)
    : LiquidCrystal(rs,rw,enable,d0,d1,d2,d3,d4,d5,d6,d7), DecodeChar(DecodeStraight) {}
  KanaLiquidCrystal(uint8_t rs, uint8_t rw, uint8_t enable, uint8_t d0, uint8_t d1, uint8_t d2, uint8_t d3)
    : LiquidCrystal(rs,rw,enable,d0,d1,d2,d3), DecodeChar(DecodeStraight) {}
  KanaLiquidCrystal(uint8_t rs, uint8_t enable, uint8_t d0, uint8_t d1, uint8_t d2, uint8_t d3)
    : LiquidCrystal(rs,enable,d0,d1,d2,d3), DecodeChar(DecodeStraight) {}
    
  // Override write function
  virtual size_t write(uint8_t ch);
  
  // Functions that turn on/off Kana decode
  void kanaOn()  { DecodeChar=DecodeUtf8;     }
  void kanaOff() { DecodeChar=DecodeStraight; }
}; // KanaLiquidCrystal


#endif // #ifndef KANALIQUIDCRYSTAL_H
